﻿using System;
using System.IO;
using System.Collections.Generic;
using FinMath.FactorAnalysis;
using FinMath.LinearAlgebra;

namespace FinMath.Tests.Performance
{
	public static class PCATest
{
		#region Tests Settings
		// Experiments parameters.
		// Following arrays must have the same size.
		static Int64[] objectsCount = {
					100, 100, 100, 100, 100, 
					500, 500, 500, 500, 500, 
					1000, 1000, 1000, 1000, 1000 };
		static Int64[] factorsCount = {
					100, 200, 300, 400, 500, 
					500, 1000, 1500, 2000, 2500, 
					1000, 2000, 3000, 4000, 5000 };
		static Int64[] repeatsCount = { 
					50, 50, 50, 50, 50, 
					10, 10, 10, 10, 10, 
					1 , 1 , 1 , 1 , 1};
		#endregion

		static Int32 lineLength = 69;

		#region FinMath Computations
		public static TimeSpan PcaFinMath(Matrix X, int repeatCount)
		{
			DateTime before = DateTime.Now;

			for (int i = 0; i < repeatCount; ++i)
			{
				PCA pca = new PCA(X);
				Matrix Y = pca.Transform(X);
			}

			return (DateTime.Now - before);
		}
		#endregion

		#region Console Utils

		public static string FormatCaption(String caption, Int32 lineLength)
		{
			Int32 l = lineLength - caption.Length - 4;
			String left = "", right;
			for (Int32 i = 0; i < l / 2; ++i)
				left += ' ';
			right = (string)left.Clone();
			if (l % 2 == 1)
				right += ' ';
			return "| " + left + caption + right + " |";
		}

		public static void DrawLine(Int32 charCount)
		{
			String line = "";
			for (int i = 0; i < charCount; ++i)
				line += "_";

			Console.WriteLine(line);
		}

		#endregion

		static void CreateTest()
		{
			if (Directory.Exists("..\\..\\Tests"))
				Directory.Delete("..\\..\\Tests", true);

			Directory.CreateDirectory("..\\..\\Tests");
			for (int i = 0; i < objectsCount.Length; ++i)
			{
				Matrix X = Matrix.Random((Int32)factorsCount[i], (Int32)objectsCount[i]);
				X.Save("..\\..\\Tests\\" + i.ToString("00") + "." + objectsCount[i] + "." + factorsCount[i] + "." + repeatsCount[i] + ".csv");
			}
		}

		static void RunTest()
		{
			DirectoryInfo dir = new DirectoryInfo("..\\..\\Tests");
			FileInfo[] tests = dir.GetFiles("*.csv");
			StreamWriter writer = new StreamWriter("..\\..\\finmath-results.csv");

			writer.WriteLine("PCA");
			DrawLine(lineLength);
			Console.WriteLine(FormatCaption("Correlation", lineLength));
			DrawLine(lineLength);
			Console.WriteLine("| Objects  |  Factors | Repeats |   Total   |  Average  |  Single   |");
			Console.WriteLine("|  count   |   count  |  count  |   Time    |  MFLOPS   | Calc Time |");
			writer.WriteLine("Objects,Factors,Repeats,FinMath Time,FinMath MFLOPS,Single calc time");
			DrawLine(lineLength);

			foreach (FileInfo test in tests)
			{
				Matrix X = Matrix.Load(test.FullName);
				String[] parameters = test.Name.Split('.');
				Int64 o = Int64.Parse(parameters[1]);
				Int64 f = Int64.Parse(parameters[2]);
				Int64 r = Int64.Parse(parameters[3]);

				TimeSpan S = PcaFinMath(X, (Int32)r);

				Double operationsCount = (o * o * f + o * o * o)* r;
				Double t = S.TotalMilliseconds / 1e+3;
				Double p = operationsCount / t / 1e+6;
				Double it = t / r;

				Console.Write(
					"| {0,8:D} | {1,8:D} | {2,7:D} | ",
					o,
					f,
					r);

				Console.WriteLine(
					"{0,9} | {1,9} | {2,9} |",
					t.ToString("0.0000"),
					Math.Ceiling(p),
					it.ToString("0.00000"));
				DrawLine(lineLength);

				writer.WriteLine(
					"{0},{1},{2},{3},{4},{5}",
					o,
					f,
					r,
					t.ToString("0.0000"),
					Math.Ceiling(p),
					it.ToString("0.00000"));
				writer.Flush();
			}
		}

		static void Main(string[] arguments)
		{
			CreateTest();
			RunTest();
		}
	}
}
